/*
 * fit_trims.C
 *
 *  Created on: Feb 10, 2015
 *      Author: lacasta
 */

#include <iostream>
#include <TH2.h>
#include <TF1.h>

class LineFitter
{
    private:
        std::vector< std::pair<double, double> > data;
        double offset, slope;

        void copy(const LineFitter &F);
    public:
        LineFitter(double offs=0, double slp=0) : offset(offs), slope(slp) {}
        LineFitter(const LineFitter &F ): offset(0), slope(0)
        {
            copy(F);
        }
        LineFitter &operator=(const LineFitter &F)
        {
            if (&F != this)
                copy(F);

            return *this;
        }
        void add(double x, double y)
        {
            data.push_back( std::pair<double, double>(x, y) );
        }
        int fit();
        double eval(double x) const { return offset + x*slope; }
        double find(double y) const { return slope!=0 ? (y-offset)/slope : 0.0 ; }
        double getSlope() const { return slope; }
        double getOffset() const  { return offset; }

};
std::ostream &operator<<(std::ostream &os, const LineFitter &F)
{
    os << F.getOffset() << " " << F.getSlope();
    return os;
}
void LineFitter::copy(const LineFitter &F)
{
    data = F.data;
    offset = F.offset;
    slope = F.slope;
}

int LineFitter::fit()
{
    std::vector< std::pair<double, double> >::iterator ip;
    double vx=0, vy=0, vxx=0, vyy=0, vxy=0, nn=0;
    double denominator = 0.0;

    for (ip=data.begin(); ip!=data.end(); ++ip)
    {
        double a=ip->first, b=ip->second;
        //printf("(%8.4f, %8.4f) ", a, b);
        if (a<=0)
            continue;
        nn++;
        vx += a;
        vy += b;
        vxx += a*a;
        vyy += b*b;
        vxy += a*b;
    }
    if (nn<=1)
    {
        std::cout << " Not enough points" << std::endl;
        return 1;
    }
    vx/=nn;
    vy/=nn;
    vxx/=nn;
    vyy/=nn;
    vxy/=nn;
    denominator = vxx - vx*vx;
    if ( denominator == 0.0 )
    {
        slope = 0.0;
        printf ("Denominator 0\nPoints follow:\n");
        for (ip=data.begin(); ip!=data.end(); ++ip)
        {
            double a=ip->first, b=ip->second;
            printf("(%8.4f, %8.4f)\n", a, b);
        }
    }
    else
        slope = (vxy-vx*vy)/(vxx-vx*vx);
    offset = vy - slope*vx;
    //printf("y=%8.3f + trim*%8.3f\n", offset, slope);
    return 0;
}

void fit_2d(TH2 *hst)
{
    int nx = hst->GetNbinsX();
    int ny = hst->GetNbinsY();


    LineFitter fitter;
    for (int ix=0; ix<nx; ix++)
    {
        double ith = 0;
        double W, W0=-1, V50p;
        double D, D0;
        int icase=0;
        int ifit = 0;

        ith = hst->GetXaxis()->GetBinCenter(ix+1);
        for (int iy=0; iy<ny; iy++)
        {
            W = hst->GetCellContent(ix+1, iy+1);
            D = hst->GetYaxis()->GetBinCenter(iy+1);

            if ( W>0.00 && icase==0)
                icase |=1;

            if ( W > 1.75 )
                 icase |= 4;

            if ( (icase & 4) && W < 0.75 )
                icase |= 8;

            if ( (icase & 8)  && W )
            {
                if (W>0.5)
                {
                    W0 = W;
                    D0 = D;
                }
                else if ( !ifit )
                {
                    V50p = (D-D0)*(0.5-W0)/(W-W0)+D0;
                    std::cout << "\t*** V50p: thrs " << ith << " val=" << V50p << std::endl
                              << "\t\t (0.25-" << W0 << ")*(" << D << "-" << D0 << ")/(" << W << "-" << W0 << ") + " << D0 << std::endl;
                    fitter.add(ith, V50p);
                    ifit++;
                }

            }
        }
    }
    fitter.fit();
    TF1 *fun = new TF1("fitter", "pol1");
    fun->SetParameters(fitter.offset, fitter.slope);
    fun->SetRange(0, 32);
    fun->Draw("same");
    std::cout << fitter << std::endl;
}


